/*
   Carte : ESP8266
   Author: phil
*/

#include <ESP8266WiFi.h>	// Librairie de getion WiFi
#include <ESP8266WebServer.h>	// Librairie des fonctions serveur Web

ESP8266WebServer server(80);	// Port du serveur (80 pour le web)

#define DEBUG	// Utile pour la mise au point

// Numero de broche ESP8266 ou Broche D6 de l'ESP8266 (PWM) vers le pilote moteur broche A-IA
#define AIA  12

// Numero de broche ESP8266 ou Broche D5 de l'ESP8266 (PWM) vers le pilote moteur broche A-IB
#define AIB  14

// Numero de broche ESP8266 ou Broche D3 de l'ESP8266 (PWM) vers le pilote moteur broche B-IA
#define BIA   0    

// Numero de broche ESP8266 ou Broche D2 de l'ESP8266 (PWM) vers le pilote moteur broche B-IB
#define BIB   4    

// Numero de broche ESP8266 ou Broche D4 vers LED Gauche
#define LEDG  2   

// Numero de broche ESP8266 ou Broche D7 vers LED Droite
#define LEDD 13  

const char *ssid = "minirobo02";	// Nom WiFi du Robot
const char *password = "minirobo02";	// Pot de passe Wifi du Robot

int speed = 1024; 	// Vitesse de rotation des moteurs


/***************************************************************************
*                                                                          *
*             Configuration de base des fonctions du robot                 *
*                                                                          *
***************************************************************************/
void setup() {

 delay(100);	// Attendre 100 millisecondes

// Pour les tests
 #ifdef DEBUG
  Serial.begin(115200);
  Serial.println();
  Serial.print("Configuring access point...");
 #endif

 WiFi.mode(WIFI_AP);	// Le robot recevra les connexions
 delay(500);		// Attendre 500 millisecondes
 WiFi.softAP(ssid, password);	// Démarrer l'interface wifi

#ifdef DEBUG
  IPAddress serverIP = WiFi.softAPIP();	// Lit l'adresse réseau (IP)
  Serial.println();	//Sauter une ligne

  Serial.print("Server IP address: ");	// Afficher L'dressse réseau
  Serial.println(WiFi.softAPIP());

  Serial.print("Server MAC address: ");	// Afficher l'adresse matériel (adresse MAC)
  Serial.println(WiFi.softAPmacAddress());
#endif

 // Configure les broches en sortie (voir #DEFINE en haut)
  pinMode(AIA, OUTPUT);
  pinMode(AIB, OUTPUT);
  pinMode(BIA, OUTPUT);
  pinMode(BIB, OUTPUT);
  pinMode(LEDG, OUTPUT);
  pinMode(LEDD, OUTPUT);

 // Configure les broches au niveau "0" (tension de sortie à zéro volts)
  digitalWrite(AIA, LOW);
  digitalWrite(AIB, LOW);
  digitalWrite(BIA, LOW);
  digitalWrite(BIB, LOW);
  digitalWrite(LEDG, LOW);
  digitalWrite(LEDD, LOW);

  // crée un lien entre le répertoire demandé, et la fonction à lancer
  server.on("/wifibot/av", mot_AV);	// Adresse pour aller en marche avant
  server.on("/wifibot/ar", mot_AR);	// Adresse pour aller en marche arrière
  server.on("/wifibot/ga", mot_GA);	// Adresse pour tourner à gauche
  server.on("/wifibot/dr", mot_DR);	// Adresse pour tourner à droite
  server.on("/wifibot/stop", mot_stop);	// Arrêt des moteurs
  server.on("/wifibot/ping", ping);	// Verification de la liaison radio WiFi

  server.begin();	// Démarre le serveur Web
  
#ifdef DEBUG
// Indique sur le port USB que le serveur à démarré
  Serial.print("Server started!");
  Serial.printf("Stations connected = %d\n", WiFi.softAPgetStationNum());  
#endif
}

/***************************************************************************
*                                                                          *
*                 Boucle principale du programme                           *
*                                                                          *
***************************************************************************/

void loop() {
  switch (WiFi.softAPgetStationNum()) {	// Vérifie si il y a des clients connectés

  case 0 :	// Pas de clients
      Serial.printf("Pas de station connect \n"); // Affiche sur le port USB: Pas de station connect
      digitalWrite(LEDG, LOW);	// éteindre la LED de gauche
      digitalWrite(LEDD, LOW);	// éteindre la LED de droite
      delay(10);	// attendre 10 milisecondes
      break;	// sortir de la boucle

      case 1 :
      Serial.printf("Une station connect \n"); // Affiche sur le port USB: Une station connect
      digitalWrite(LEDG, !digitalRead(LEDG));	// inverser l'état de la LED gauche
      server.handleClient();	// fait le lien entre le client et la page demandé
      break;	// sortir de la boucle

      case 2 :
      Serial.printf("deux station connect \n"); // Affiche sur le port USB: deux station connect
      digitalWrite(LEDD, !digitalRead(LEDD)); // inverser l'état de la LED droite
      mot_stop();	// Trop de clients. Arrêt du moteur
      delay(10);	// attendre 10 milisecondes
      break;	// sortir de la boucle
  }
}

void mot_stop() {

  server.send(200, "text/plain", "STOP" ); // Affiche sur l'écran client le message STOP
  setMotor(0, 0, 0, 0); // Arrête les moteur
  /*
  digitalWrite(AIA, LOW);    // Exemple de commande des broches
  digitalWrite(AIB, LOW);
  digitalWrite(BIA, LOW);
  digitalWrite(BIB, LOW);
  */
#ifdef DEBUG
  // Serial.println("STOP!!");
#endif
}

void ping() {
  server.send(200, "text/plain", "ok" ); // Affiche sur l'écran client le message ok
#ifdef DEBUG
  Serial.println("PING !"); // Affiche sur le port USB: PING !
#endif
}

void mot_GA() {
  server.send(200, "text/plain", "A gauche" ); // Affiche sur l'écran client le message A gauche
  setMotor(0, speed, speed, 0); // AIB=0  AIA=Vitesse  BIB=Vitesse  BIA=0
#ifdef DEBUG
  Serial.println("A gauche"); // Affiche sur le port USB: A gauche
#endif
}

void mot_DR() {
  server.send(200, "text/plain", "A droite" ); // Affiche sur l'écran client le message A droite
  setMotor(speed, 0, 0, speed); // AIB=Vitesse  AIA=0  BIB=0  BIA=Vitesse
#ifdef DEBUG
  Serial.println("A droite"); // Affiche sur le port USB: A droite
#endif
}

void mot_AR() {
  server.send(200, "text/plain", "En arriere" ); // Affiche sur l'écran client le message En arriere
  setMotor(0, speed, 0, speed); // AIB=0  AIA=Vitesse  BIB=0  BIA=Vitesse
#ifdef DEBUG
  Serial.println("En arriere "); // Affiche sur le port USB: En arriere
#endif
}

void mot_AV() {
  server.send(200, "text/plain", "En avant" ); // Affiche sur l'écran client le message En avant
  setMotor(speed, 0, speed, 0); // AIB=Vitesse  AIA=0  BIB=Vitesse  BIA=0
#ifdef DEBUG
  Serial.println("En avant"); // Affiche sur le port USB: En avant
#endif
}

void setMotor(int vAIB, int vAIA, int vBIB int vBIA) {

// Positionne les valeurs de commande moteur sur les broches
  analogWrite(AIA, vAIA);
  analogWrite(AIB, vAIB);
  analogWrite(BIA, vBIA);
  analogWrite(BIB, vBIB);

  delay(100); // Attendre 100 millisecondes

// Arrête les moteurs
  analogWrite(AIA, 0);
  analogWrite(AIB, 0);
  analogWrite(BIA, 0);
  analogWrite(BIB, 0);
}
